/*
 * Copyright (C) 2012 Chris Burdess <dog@gnu.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const Extension = imports.ui.extensionSystem.extensions["biff@bluezoo.org"];
const md5 = Extension.md5;

function CramMD5(username, password) {
    this.init(username, password);
}

CramMD5.prototype = {

init : function(username, password) {
    this.username = sasl_string_to_array(username);
    this.password = sasl_string_to_array(password);
    this.complete = false;
},

get_name : function() {
    return "CRAM-MD5";
},

is_plaintext : function() {
    return false;
},

has_initial_response : function() {
    return false;
},

evaluate_challenge : function(challenge) {
    let digest = this.hmac_md5(this.password, challenge);
    let ret = [];
    sasl_array_concat(ret, this.username);
    ret.push(0x20);
    sasl_array_concat(ret, digest);
    this.complete = true;
    return ret;
},

is_complete : function() {
    return this.complete;
},

hmac_md5 : function(key, text) {
    let k_ipad = [];
    let k_opad = [];
    let digest;    
    if (key.length > 64) {
        key = sasl_array_md5(key);
    }
    for (let i = 0; i < 64; i++) {
        let val = 0;
        if (i < key.length) {
            val = key[i];
        }
        k_ipad.push(val ^ 0x36);
        k_opad.push(val ^ 0x5c);
    }
    digest = sasl_array_md5(sasl_array_concat(k_ipad, text));
    digest = sasl_array_md5(sasl_array_concat(k_opad, digest));
    return digest;
}

};

function Plain(username, password) {
    this.init(username, password);
}

Plain.prototype = {

init : function(username, password) {
    this.username = sasl_string_to_array(username);
    this.password = sasl_string_to_array(password);
    this.complete = false;
},

get_name : function() {
    return "PLAIN";
},

is_plaintext : function() {
    return true;
},

has_initial_response : function() {
    return true;
},

evaluate_challenge : function(challenge) {
    let ret = [];
    sasl_array_concat(ret, this.username);
    ret.push(0);
    sasl_array_concat(ret, this.username);
    ret.push(0);
    sasl_array_concat(ret, this.password);
    this.complete = true;
    return ret;
},

is_complete : function() {
    return this.complete;
}

};

function sasl_string_to_array(string) {
    let ret = [];
    for (let i = 0; i < string.length; i++) {
        let c = string.charCodeAt(i);
        ret.push(c); // this should probably handle non-ASCII values better
    }
    return ret;
}

function sasl_array_concat(array1, array2) {
    for (let i = 0; i < array2.length; i++) {
        array1.push(array2[i]);
    }
    return array1;
}

function sasl_array_md5(array) {
    let w_in = [], wlen = array.length >> 2;
    for (let i = 0; i < wlen; i++) {
        w_in.push(0);
    }
    for (let i = 0; i < array.length * 8; i += 8) {
        w_in[i >> 5] |= (array[i / 8] & 0xff) << (i % 32);
    }
    let w_out = md5.binl_md5(w_in, array.length * 8);
    let ret = [];
    for (let i = 0; i < w_out.length * 32; i += 8) {
        ret.push((w_out[i >> 5] >>> (i % 32)) & 0xff);
    }
    return ret;
}

